<?php

namespace Drupal\group_content_default_menu\Plugin\Group\Relation;

use Drupal\group\Entity\GroupInterface;
use Drupal\group\Plugin\Group\Relation\GroupRelationBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\group\Entity\GroupRelationshipType;
use Drupal\link\LinkItemInterface;
use Drupal\Core\Entity\Element\EntityAutocomplete;

/**
 * Provides a content enabler for group content default menu.
 *
 * @GroupRelationType(
 *   id = "group_content_default_menu",
 *   label = @Translation("Group Content Default Menu"),
 *   description = @Translation("Adds a default menu for groups that have a group content menu plugin installed."),
 *   entity_type_id = "group_content_menu",
 *   entity_access = FALSE,
 * )
 */


class GroupContentDefaultMenu extends GroupRelationBase
{
    //   *   deriver = "Drupal\gnode\Plugin\Group\Relation\GroupNodeDeriver",
    //   *   handlers = {
    //  *     "access" = "Drupal\group\Plugin\GroupContentAccessControlHandler",
    //  *     "permission_provider" = "Drupal\group\Plugin\GroupContentPermissionProvider",
    //  *   }
    /**
     * {@inheritdoc}
     */
    public function getGroupOperations(GroupInterface $group)
    {
        return [];
    }

    /**
     * {@inheritdoc}
     */
    public function defaultConfiguration()
    {
        $config = parent::defaultConfiguration();
        $config['entity_cardinality'] = 1;
        $config['group_cardinality'] = 1;

        $config['links'] = [];
        return $config;
    }

    /**
     * Get the plugin key for the installed group_content_menu plugin
     * @return int|string|null
     */
    private function get_group_content_menu_plugin_key()
    {
        $group_type = $this->getGroupType();
        foreach (array_keys($group_type->getInstalledPlugins()->getConfiguration()) as $plugin_key) {
            if (substr($plugin_key, 0, strlen('group_content_menu:')) == 'group_content_menu:') {
                return $plugin_key;
            }
        }
        return null;
    }

    /**
     * Get the GroupRelationshipType associated with the input menu plugin id
     * @param $plugin_id
     * @return \Drupal\group\Entity\GroupRelationshipTypeInterface|null
     */
    private function getGroupContentTypeForMenuPlugin($plugin_id)
    {
        $group_type_id = $this->getGroupTypeId();
        foreach (GroupRelationshipType::loadByPluginId($plugin_id) as $groupRelationshipType) {
            if ($groupRelationshipType->getGroupTypeId() === $group_type_id) {
                return $groupRelationshipType;
            }
        }
        return null;
    }

    /**
     * AJAX callback for adding a link.
     *
     * @param array $form
     * @param FormStateInterface $form_state
     * @return mixed
     */
    public function add_link_callback(array &$form, FormStateInterface $form_state)
    {
        return $form['links'];
    }

    /**
     * AJAX callback for deleting a link.
     *
     * @param array $form
     * @param FormStateInterface $form_state
     * @return mixed
     */
    public function delete_link_callback(array &$form, FormStateInterface $form_state)
    {
        return $form['links'];
    }

    /**
     * Adopted from Drupal core:
     *      https://git.drupalcode.org/project/drupal/blob/8.8.0/core/modules/link/src/Plugin/Field/FieldWidget/LinkWidget.php#L186-203
     * Gets the user-entered string as a URI.
     *
     * The following two forms of input are mapped to URIs:
     * - entity autocomplete ("label (entity id)") strings: to 'entity:' URIs;
     * - strings without a detectable scheme: to 'internal:' URIs.
     *
     * This method is the inverse of ::getUriAsDisplayableString().
     *
     * @param string $string
     *   The user-entered string.
     *
     * @return string
     *   The URI, if a non-empty $uri was passed.
     *
     * @see static::getUriAsDisplayableString()
     */
    protected static function getUserEnteredStringAsUri($string) {
        // By default, assume the entered string is an URI.
        $uri = trim($string);

        // Detect entity autocomplete string, map to 'entity:' URI.
        $entity_id = EntityAutocomplete::extractEntityIdFromAutocompleteInput($string);
        if ($entity_id !== NULL) {
            // @todo Support entity types other than 'node'. Will be fixed in
            //    https://www.drupal.org/node/2423093.
            $uri = 'entity:node/' . $entity_id;
        }
        // Support linking to nothing.
        elseif (in_array($string, ['<nolink>', '<none>'], TRUE)) {
            $uri = 'route:' . $string;
        }
        // Detect a schemeless string, map to 'internal:' URI.
        elseif (!empty($string) && parse_url($string, PHP_URL_SCHEME) === NULL) {
            // @todo '<front>' is valid input for BC reasons, may be removed by
            //   https://www.drupal.org/node/2421941
            // - '<front>' -> '/'
            // - '<front>#foo' -> '/#foo'
            if (strpos($string, '<front>') === 0) {
                $string = '/' . substr($string, strlen('<front>'));
            }
            $uri = 'internal:' . $string;
        }

        return $uri;
    }

    /**
     * Adopted from Drupal core:
     *  https://git.drupalcode.org/project/drupal/blob/8.8.0/core/modules/link/src/Plugin/Field/FieldWidget/LinkWidget.php#L186-203
     * Form element validation handler for the 'uri' element.
     *
     * Disallows saving inaccessible or untrusted URLs.
     */
    public static function validateUriElement($element, $form_state, $form) {
        $uri = static::getUserEnteredStringAsUri($element['#value']);
        $form_state->setValueForElement($element, $uri);

        // If getUserEnteredStringAsUri() mapped the entered value to a 'internal:'
        // URI , ensure the raw value begins with '/', '?' or '#'.
        // @todo '<front>' is valid input for BC reasons, may be removed by
        //   https://www.drupal.org/node/2421941
        if (parse_url($uri, PHP_URL_SCHEME) === 'internal' && !in_array($element['#value'][0], ['/', '?', '#'], TRUE) && substr($element['#value'], 0, 7) !== '<front>' && substr($element['#value'], 0, 9) !== 'internal:') {
            $form_state->setError($element, t('Manually entered paths should start with one of the following characters: / ? #'));
            return;
        }
    }

    /**
     * {@inheritdoc}
     */
    public function buildConfigurationForm(array $form, FormStateInterface $form_state)
    {

        $form = parent::buildConfigurationForm($form, $form_state);
        $configuration = $this->getConfiguration();
        $default_menu_links = $configuration['links'];// ?? [];

        // Disable the entity cardinality field as the functionality of this module
        // relies on a cardinality of 1. We don't just hide it, though, to keep a UI
        // that's consistent with other content enabler plugins.
        $info = $this->t("This field has been disabled by the plugin to guarantee the functionality that's expected of it.");
        $form['entity_cardinality']['#disabled'] = TRUE;
        $form['entity_cardinality']['#description'] .= '<br /><em>' . $info . '</em>';
        $form['group_cardinality']['#disabled'] = TRUE;
        $form['group_cardinality']['#description'] .= '<br /><em>' . $info . '</em>';

        if (!$this->get_group_content_menu_plugin_key()) {
            $form['error_message'] = [
                '#type' => 'markup',
                '#markup' => $this->t('Please install a <em>group content menu</em> plugin for this group type, before installing this plugin.'),
            ];
            return $form;
        }

        $group_type = $this->getGroupType();
        

        $form['links'] = [
            '#type' => 'container',
            '#tree' => TRUE,
            '#attributes' => ['id' => 'links-wrapper']
        ];

        // Render the default menu links for the chosen group type.
        $group_type_links = $form_state->get("links");
        if (!isset($group_type_links)) {
            $group_type_links = $default_menu_links;
            if (!$group_type_links) {
                $group_type_links = [];
            }
        }

        $form_state->set("links", $group_type_links);

        $current_link_index = 0;
        foreach ($group_type_links as $link_id => $link) {
            $current_link_index += 1;

            if ($link['details']['menu_link_title']) {
                $link_title = $link['details']['menu_link_title'];
            } else {
                $link_title = "Link $current_link_index";
            }

            $form['links'][$link_id] = [
                '#type' => 'container',
                '#tree' => TRUE,
                // '#attributes' => ['id' => "$link_id-wrapper"]
            ];

            $form['links'][$link_id]['details'] = [
                '#type' => 'details',
                '#title' => $this->t($link_title)
            ];
            $form['links'][$link_id]['details']["menu_link_title"] = [
                // '#name' => "link_menu_link_title_$link_id",
                '#type' => 'textfield',
                '#title' => $this->t('Menu link title'),
                '#default_value' => $link['details']['menu_link_title'] ?? NULL,
                // '#value' => $link['menu_link_title'] ?? NULL,
                '#description' => "The text to be used for this link in the menu.",
            ];
            $form['links'][$link_id]['details']["link"] = [
                // '#name' => "link_$link_id",
                '#type' => 'entity_autocomplete',
                '#target_type' => 'node',
                '#title' => $this->t('Link'),
                '#default_value' => $link['details']['link'] ?? NULL,
                // '#value' => $link['link'] ?? NULL,
                '#element_validate' => [[get_called_class(), 'validateUriElement']],
                '#maxlength' => 2048,
                '#description' => "The location this menu link points to.",
                '#link_type' => LinkItemInterface::LINK_GENERIC,
                '#attributes' => [
                    'data-autocomplete-first-character-blacklist' => '/#?'
                ],
                '#process_default_value' => FALSE
            ];
            $form['links'][$link_id]['details']["weight"] = array(
                '#type' => 'number',
                // '#name' => "link_weight_$link_id",
                '#title' => $this
                    ->t('Weight'),
                '#default_value' => $link['details']['weight'] ?? NULL,
                // '#value' => $link['weight'] ?? NULL,
            );
            $form['links'][$link_id]['details']['delete'] = [
                '#type' => 'button',
                '#name' => "delete_link_button_$link_id",
                '#value' => $this->t('Delete'),
                '#ajax' => [
                    'callback' => [$this, 'delete_link_callback'],
                    'wrapper' => 'links-wrapper',
                    'event' => 'click',
                ],
            ];
        }

        $form['links']['add_link'] = [
            '#type' => 'button',
            '#name' => 'add_link_button',
            '#value' => $this->t('Add link'),
            '#ajax' => [
                'callback' => [$this, 'add_link_callback'],
                'wrapper' => 'links-wrapper',
                'event' => 'click',
            ],
        ];


        $form['token_help'] = [
            '#theme' => 'token_tree_link',
            '#token_types' => 'all'
        ];


        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public function calculateDependencies()
    {
        $dependencies = parent::calculateDependencies();
        $dependencies['module'][] = 'group_content_default_menu';
        // $dependencies['config'][] = 'group_content_default_menu.settings';
        return $dependencies;
    }

    public function validateConfigurationForm(array &$form, FormStateInterface $form_state)
    {
        $configuration = $this->getConfiguration();
        $default_menu_links = $configuration['links'];
        $trigger = $form_state->getTriggeringElement();

        // Handle adding a new link
        if ($trigger['#name'] == 'add_link_button') {
            $group_type_links = $form_state->get("links");
            if (!$group_type_links) {
                $group_type_links = $default_menu_links;
                if (!$group_type_links) {
                    $group_type_links = [];
                }
            }
            $new_link_id = count($group_type_links); //"$selected_group_type" . "_" . count($group_type_links); //uniqid(); //count($group_type_links); //uniqid();
            $group_type_links[$new_link_id] = [];
            $form_state->set("links", $group_type_links);
            $form_state->setRebuild(TRUE);
            return;
        }
        // Handle deleting an existing link
        else if (substr($trigger['#name'], 0, 19) == "delete_link_button_") {
            $group_type_links = $form_state->get("links");
            if (!$group_type_links) {
                $group_type_links = $default_menu_links;
                if (!$group_type_links) {
                    $group_type_links = [];
                }
            }

            $link_id_to_delete = substr($trigger['#name'], 19);

            unset($group_type_links[$link_id_to_delete]);

            if (!$group_type_links) {
                $group_type_links = [];
            }

            $form_state->set("links", $group_type_links);
            $form_state->setRebuild();
        }
        // Handle overall form validation before submission.
        else {
            if (!$this->get_group_content_menu_plugin_key()) {
                $form_state->setErrorByName('error_message', $this->t('Please install a <em>group content menu</em> plugin for this group type, before installing this plugin.'));
                $form_state->setRebuild();
                return;
            }    

            // Ensure that all menu link titles and entity autocompletes are valid and filled in
            foreach ($form_state->getValue(['links']) as $link_id => $link_item) {
                if ($link_id === "add_link") {
                    continue;
                }

                $link_item_menu_title = $link_item['details']['menu_link_title'];
                $link_item_link = $link_item['details']['link'];


                if (!$link_item_menu_title) {
                    $form_state->setErrorByName(
                        implode('][', ['links', $link_id, 'details', 'menu_link_title']),
                        "Menu item title is required.");
                }
                if (!$link_item_link) {
                    $form_state->setErrorByName(
                        implode('][', ['links', $link_id, 'details', 'link']),
                        "Menu item link is required."
                    );
                }
            }
        }
    }



    /**
     * {@inheritdoc}
     */
    public function submitConfigurationForm(array &$form, $form_state)
    {
        if (!$this->get_group_content_menu_plugin_key()) {
            $form_state->setErrorByName('error_message', $this->t('Please install a <em>group content menu</em> plugin for this group type, before installing this plugin.'));
            $form_state->setRebuild();
            return;
        }

        $selected_group_type_links = $form_state->get("links");

        $final_group_type_links = [];

        foreach (array_keys($selected_group_type_links) as $link_id) {
            // $link = [
            //     'id' => $link_id,
            //     'menu_link_title' => $form_state->getValue(array("links", $link_id, "details", "menu_link_title")),
            //     'link' => $form_state->getValue(array("links", $link_id, "details", "link")),
            //     'weight' => $form_state->getValue(array("links", $link_id, "details", "weight")),
            // ];
            // $final_group_type_links[] = $link;
            $final_group_type_links[$link_id] = [
                'details' => [
                    'menu_link_title' => $form_state->getValue(array("links", $link_id, "details", "menu_link_title")),
                    'link' => $form_state->getValue(array("links", $link_id, "details", "link")),
                    'weight' => $form_state->getValue(array("links", $link_id, "details", "weight")),
                ]
            ];    
        }
        
        // $this->configuration['default_menu_links'] = $final_group_type_links;
        $this->setConfiguration(['links' => $final_group_type_links]);
        // dpm("Setting configuration to:");
        // dpm($this->configuration);
        parent::submitConfigurationForm($form, $form_state);
    }
}
